/* -*- mode: c++; tab-width: 4; c-basic-offset: 4 -*- */
/*
 * Copyright (C) 2005 Atmark <atmarkat _AT_ msn _DOT_ com>
 *                    AGAWA Koji <i _AT_ atty _DOT_ jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef QT4
#include <QGroupBox>
#include <QHBoxLayout>
#include <QButtonGroup>
#else
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qvbox.h>
#include <qhbuttongroup.h>
#endif

#include <qlayout.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
#include <qtextcodec.h>
#include <qfileinfo.h>
#include "simplefiledialog.h"
#include "mplayerconfig.h"

/*! configの値を初期値にして構築
*/
MPlayerConfig::MPlayerConfig(Configuration& configuration, QWidget *parent)
	: ConfigView(configuration, parent)
{
}

/*! \a config を元にして各Widgetの状態を設定します。
 */
void MPlayerConfig::setStatesByConfiguration(const Configuration& config)
{
	ConfigView::setStatesByConfiguration(config);

	if (config.machineCategory() == Configuration::MACHINE_GENERIC) {
		isOverlayEnabled_->setChecked(false);
		isOverlayEnabled_->hide();
	} else {
		isOverlayEnabled_->setChecked(config.isOverlayEnabled);
		isOverlayEnabled_->show();
	}
	isDoubleBufferingEnabled_->setChecked(config.isDoubleBufferingEnabled);
#ifdef QT4
	reinterpret_cast<QRadioButton *>(framedropMethod_->button(config.framedropMethod))->setChecked(true);
#else
	reinterpret_cast<QRadioButton *>(framedropMethod_->find(config.framedropMethod))->setChecked(true);
#endif
	isAdditionalOptionsEnabled_->setChecked(config.isAdditionalOptionsEnabled);

	isCacheEnabled_->setChecked(config.isCacheEnabled);
	cacheSize_->setValue(config.cacheSize);
	isPrefetchFileEnabled_->setChecked(config.isPrefetchFileEnabled);

	additionalOptions_->setText(config.additionalOptions);
	additionalOptions_->setEnabled(config.isAdditionalOptionsEnabled);
	mplayerPath_->setText(config.mplayerPath);
}

/*! 使用するWidgetを確保する
*/
void MPlayerConfig::createWidgets()
{
	ConfigView::createWidgets();

	QVBoxLayout *layout = new QVBoxLayout(widget());
	layout->setSpacing(4);

#ifdef QT4
	QGroupBox *group = new QGroupBox(tr("MPlayer settings"), widget());
	QVBoxLayout *groupLayout = new QVBoxLayout(group);
#else
	QVGroupBox *group = new QVGroupBox(tr("MPlayer settings"), widget());
#endif

	layout->addWidget(group);

	isOverlayEnabled_ = new QCheckBox(tr("Enable video overlay (BVDD / ATI W100)"), group);
	connect(isOverlayEnabled_, SIGNAL(toggled(bool)),
			this, SLOT(setOverlayEnabled(bool)));

	isDoubleBufferingEnabled_ = new QCheckBox(tr("Enable double buffering (for OSD and smoother playback)"), group);
	connect(isDoubleBufferingEnabled_, SIGNAL(toggled(bool)),
			this, SLOT(setDoubleBufferingEnabled(bool)));

	// -------------------------------------------------------------

	QWidget *hbox = new QWidget(group);
	QHBoxLayout *hboxlayout = new QHBoxLayout(hbox);
	hboxlayout->setMargin(0);
	hboxlayout->setSpacing(4);

	isCacheEnabled_ = new QCheckBox(tr("Enable file cache:"), hbox);
	hboxlayout->addWidget(isCacheEnabled_);
	connect(isCacheEnabled_, SIGNAL(toggled(bool)),
			this, SLOT(setCacheEnabled(bool)));

#ifdef QT4
	cacheSize_ = new QSpinBox(hbox);
	cacheSize_->setMinimum(128);
	cacheSize_->setMaximum(16384);
	cacheSize_->setSingleStep(128);
#else
	cacheSize_ = new QSpinBox(128, 16384, 128, hbox);
#endif
	hboxlayout->addWidget(cacheSize_);
	cacheSize_->setSuffix(" KiB");
	connect(cacheSize_, SIGNAL(valueChanged(int)), this, SLOT(setCacheSize(int)));

	// -------------------------------------------------------------

	isPrefetchFileEnabled_ = new QCheckBox(tr("Enable file prefetching"), group);
	connect(isPrefetchFileEnabled_, SIGNAL(toggled(bool)),
			this, SLOT(setPrefetchFileEnabled(bool)));

#ifdef QT4
	QGroupBox *framedropMethodGroupBox = new QGroupBox(tr("Framedrop method"), group);
	QRadioButton *framedropMethodNone = new QRadioButton(tr("None"), framedropMethodGroupBox);
	QRadioButton *framedropMethodNormal = new QRadioButton(tr("Normal"), framedropMethodGroupBox);
	QRadioButton *framedropMethodHard = new QRadioButton(tr("Hard (may crash MPlayer)"), framedropMethodGroupBox);

	QHBoxLayout *framedropMethodLayout = new QHBoxLayout(framedropMethodGroupBox);
	framedropMethodLayout->setSpacing(0);
	framedropMethodLayout->addWidget(framedropMethodNone);
	framedropMethodLayout->addWidget(framedropMethodNormal);
	framedropMethodLayout->addWidget(framedropMethodHard);

	framedropMethod_ = new QButtonGroup(framedropMethodGroupBox);
	framedropMethod_->addButton(framedropMethodNone, Configuration::FRAMEDROP_NONE);
	framedropMethod_->addButton(framedropMethodNormal, Configuration::FRAMEDROP_NORMAL);
	framedropMethod_->addButton(framedropMethodHard, Configuration::FRAMEDROP_HARD);
	framedropMethod_->setExclusive(true);
	connect(framedropMethod_, SIGNAL(buttonClicked(int)), this, SLOT(setFramedropMethod(int)));
#else
	framedropMethod_ = new QHButtonGroup(tr("Framedrop method"), group);
	framedropMethod_->setExclusive(true);
	QRadioButton *framedropMethodNone = new QRadioButton(tr("None"), framedropMethod_);
	QRadioButton *framedropMethodNormal = new QRadioButton(tr("Normal"), framedropMethod_);
	QRadioButton *framedropMethodHard = new QRadioButton(tr("Hard (may crash MPlayer)"), framedropMethod_);
	connect(framedropMethod_, SIGNAL(clicked(int)), this, SLOT(setFramedropMethod(int)));
#endif

	isAdditionalOptionsEnabled_ = new QCheckBox(tr("Use additional options:"), group);
	connect(isAdditionalOptionsEnabled_, SIGNAL(toggled(bool)),
			this, SLOT(setAdditionalOptionsEnabled(bool)));
	additionalOptions_ = new QLineEdit(group);
	connect(additionalOptions_, SIGNAL(textChanged(const QString&)),
			this, SLOT(setAdditionalOptions(const QString&)));

#ifdef QT4
	groupLayout->addWidget(isOverlayEnabled_);
	groupLayout->addWidget(isDoubleBufferingEnabled_);
	groupLayout->addWidget(hbox);
	groupLayout->addWidget(isPrefetchFileEnabled_);
	groupLayout->addWidget(framedropMethodGroupBox);
	groupLayout->addWidget(isAdditionalOptionsEnabled_);
	groupLayout->addWidget(additionalOptions_);
#endif

	// ----------------

#ifdef QT4
	QGroupBox *pathGroup = new QGroupBox(tr("MPlayer location"), widget());
	QHBoxLayout *pathGroupLayout = new QHBoxLayout(pathGroup);
#else
	QHGroupBox *pathGroup = new QHGroupBox(tr("MPlayer location"), widget());
#endif

	layout->addWidget(pathGroup);

	mplayerPath_ = new QLineEdit(pathGroup);
	connect(mplayerPath_, SIGNAL(textChanged(const QString&)),
			this, SLOT(setMPlayerPath(const QString&)));
	QPushButton *openButton = new QPushButton(tr("..."), pathGroup);
	connect(openButton, SIGNAL(clicked()), this, SLOT(execMPlayerPathDialog()));

#ifdef QT4
	pathGroupLayout->addWidget(mplayerPath_);
	pathGroupLayout->addWidget(openButton);
#endif
}

void MPlayerConfig::execMPlayerPathDialog()
{
	QString selectedFilename = SimpleFileDialog::getOpenFileName(
		QFileInfo(mplayerPath_->text()).filePath(),
		"All Files (*.*)",
		this,
		"locate mplayer dialog",
		tr("Locate MPlayer binary")
	);

	if (!selectedFilename.isNull())
		mplayerPath_->setText(selectedFilename);
}
